function ARI_DruidAssist()
	if ARI_HealDruid() then
		return true
	end

	local leader = ARI_FindLeader()

	if ARI_UseCC ~= nil then
		if ARI_IsActionValid(ARI_LOC_HIBERNATE) then
			if ARI_GetRaidTarget(ARI_UseCC) then
				if not ARI_UnitHasDebuff("target", ARI_LOC_HIBERNATE) then
					if ARI_CastAction(ARI_LOC_HIBERNATE) then
						return true
					end
				end
				ARI_ResetUnitAuraOne("target")
				ClearTarget()
			end
		end
	end

	AssistUnit(leader)

	if not UnitCanAttack("player", "target") then
		ClearTarget()
		return false
	end

	if UnitIsDead("target") then
		ClearTarget()
		return false
	end

	local manaPercent = UnitMana("player") / UnitManaMax("player")
	if ARI_UseSpells then
		if manaPercent > 0.20 then
			if ARI_UseDots then
				if ARI_IsActionValid(ARI_LOC_MOONFIRE) then
					if not ARI_IsImmune(ARI_LOC_MOONFIRE, UnitName("target")) then
						if not ARI_UnitHasDebuff("target", ARI_LOC_MOONFIRE) then
							if ARI_CastAction(ARI_LOC_MOONFIRE) then
								return true
							end
						end
					end
				end

				if ARI_IsActionValid(ARI_LOC_INSECT_SWARM) then
					if not ARI_IsImmune(ARI_LOC_INSECT_SWARM, UnitName("target")) then
						if not ARI_UnitHasDebuff("target", ARI_LOC_INSECT_SWARM) then
							if ARI_CastAction(ARI_LOC_INSECT_SWARM) then
								return true
							end
						end
					end
				end
			end

			if ARI_IsActionValid(ARI_LOC_STARFIRE) then
				if not ARI_IsImmune(ARI_LOC_STARFIRE, UnitName("target")) then
					if ARI_CastAction(ARI_LOC_STARFIRE) then
						return true
					end
				end
			end

			if ARI_IsActionValid(ARI_LOC_WRATH) ~= nil then
				if not ARI_IsImmune(ARI_LOC_WRATH, UnitName("target")) then
					if ARI_CastAction(ARI_LOC_WRATH) then
						return true
					end
				end
			end
		end
	end

	return false
end

function ARI_HealDruid()
	if UnitIsDead("player") then
		return true
	end

	-- force view and refresh spell slots
	SetView(5)
	ARI_CheckActionSlots()
	ARI_ResetUnitAura()

	if ARI_UnitHasDebuff("player", ARI_LOC_MANDOKIR_THREATENING_GAZE) then
		if ARI_PlayerIsCasting() then
			SpellStopCasting()
		end
		return true
	end

	if ARI_CheckManaConserve() then
		return true
	end

	-- check combat
	local inCombat = UnitAffectingCombat("player")
	if inCombat then
		-- check mana (in combat)
		local manaPercent = UnitMana("player") / UnitManaMax("player")
		if ARI_UsePotions then
			if manaPercent < 0.66 then
				if ARI_UnitCastAction("player", ARI_LOC_MAJOR_MANA_POTION) then
					--return true
				end
			end
		end
		if manaPercent < 0.25 then
			if ARI_UnitCastAction("player", ARI_LOC_INNERVATE) then
				return true
			end
		end
	end

	-- refresh units in party
	ARI_UpdateUnitNames()

	local focus = ARI_FindFocus()
	local leader = ARI_FindLeader()

	-- check debuffs (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealDruid_CheckDebuffsExt(focus) then
			return true
		end
	end
	if ARI_HealDruid_CheckDebuffsExt(leader) then
		return true
	end

	-- check health (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealDruid_CheckHealthExt(focus) then
			return true
		end
	end
	if ARI_HealDruid_CheckHealthExt(leader) then
		return true
	end

	-- check debuffs (others)
	for _, unit in pairs(ARI_UnitNamesBuff) do
		if ARI_HealDruid_CheckDebuffs(unit) then
			return true
		end
	end
	
	-- check health (others)
	for _, unit in pairs(ARI_UnitNamesHeal) do
		if ARI_HealDruid_CheckHealth(unit) then
			return true
		end
	end

	-- check buffs (out of combat)
	if not inCombat then
		local deadCount = table.getn(ARI_UnitNamesDead)
		if deadCount == 0 then
			for _, unit in pairs(ARI_UnitNamesBuff) do
				if ARI_HealDruid_CheckBuffs(unit) then
					return true
				end
			end
		end
	end
	return false
end

function ARI_HealDruid_CheckHealthExt(unit)
	if not UnitIsDead(unit) then
		local inRange = true
		if ARI_CheckInteractDistanceExtension then
			-- check heal distance (40m)
			inRange = CheckInteractDistance(unit, 7)
		end
		if inRange then
			ARI_HealDruid_CheckHealth(unit)
		end
	end
end

function ARI_HealDruid_CheckHealth(unit)
	local healthPercent = UnitHealth(unit) / UnitHealthMax(unit)
	if healthPercent <= 0.4 then
		if ARI_IsActionValid(ARI_LOC_REGROWTH) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_REGROWTH) then
				if ARI_UnitCastAction(unit, ARI_LOC_REGROWTH) then
					ARI_SetManaConserve(unit, 0.7)
					return true
				end
			end
		end
	end

	if healthPercent <= 0.7 then
		if ARI_IsActionValid(ARI_LOC_HEALING_TOUCH) then
			if ARI_UnitCastAction(unit, ARI_LOC_HEALING_TOUCH) then
				ARI_SetManaConserve(unit, 0.9)
				return true
			end
		end
	end

	if ARI_UseHots then
		if healthPercent <= 0.8 then
			if ARI_IsActionValid(ARI_LOC_REJUVENATION) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_REJUVENATION) then
					if ARI_UnitCastAction(unit, ARI_LOC_REJUVENATION) then
						return true
					end
				end
			end
		end
	end

	return false
end

function ARI_HealDruid_CheckDebuffsExt(unit)
	if not UnitIsDead(unit) then
		-- check follow/buff distance (28m/30m)
		if CheckInteractDistance(unit, 4) then
			ARI_HealDruid_CheckDebuffs(unit)
		end
	end
end

function ARI_HealDruid_CheckDebuffs(unit)
	if ARI_IsActionValid(ARI_LOC_CURE_POISON) or ARI_IsActionValid(ARI_LOC_ABOLISH_POISON) then
		if ARI_UnitHasDebuffType(unit, ARI_LOC_POISON ) then
			if ARI_IsActionValid(ARI_LOC_ABOLISH_POISON) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_ABOLISH_POISON) then
					if ARI_UnitCastAction(unit, ARI_LOC_ABOLISH_POISON) then
						return true
					end
				end
			end
			if ARI_IsActionValid(ARI_LOC_CURE_POISON) then
				if ARI_UnitCastAction(unit, ARI_LOC_CURE_POISON) then
					return true
				end
			end
		end
	end

	if ARI_IsActionValid(ARI_LOC_REMOVE_CURSE) then
		if ARI_UnitHasDebuffType(unit, ARI_LOC_CURSE) then
			if ARI_UnitCastAction(unit, ARI_LOC_REMOVE_CURSE) then
				return true
			end
		end
	end

	return false
end

function ARI_HealDruid_CheckBuffs(unit)
	if not ARI_UnitHasBuff(unit, ARI_LOC_PHASE_SHIFT) then
		if ARI_IsActionValid(ARI_LOC_MARK_OF_THE_WILD) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_GIFT_OF_THE_WILD) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_MARK_OF_THE_WILD) then
					if ARI_UnitCastAction(unit, ARI_LOC_GIFT_OF_THE_WILD) then
						return true
					end
					if ARI_UnitCastAction(unit, ARI_LOC_MARK_OF_THE_WILD) then
						return true
					end
				end
			end
		end

		if ARI_IsActionValid(ARI_LOC_THORNS) then
			if UnitClass(unit) == ARI_LOC_WARRIOR then
				if not ARI_UnitHasBuff(unit, ARI_LOC_THORNS) then
					if ARI_UnitCastAction(unit, ARI_LOC_THORNS) then
						return true
					end
				end
			elseif UnitClass(unit) == ARI_LOC_DRUID then
				if ARI_UnitHasBuff(unit, ARI_LOC_DIRE_BEAR_FORM) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_THORNS) then
						if ARI_UnitCastAction(unit, ARI_LOC_THORNS) then
							return true
						end
					end
				end
			end
		end
	end

	return false
end