function ARI_MageAssist()
	if ARI_HealMage() then
		return true
	end

	local leader = ARI_FindLeader()

	if ARI_UseCC ~= nil then
		if ARI_IsActionValid(ARI_LOC_POLYMORPH) then
			if ARI_GetRaidTarget(ARI_UseCC) then
				if not ARI_UnitHasDebuff("target", ARI_LOC_POLYMORPH) then
					if ARI_CastAction(ARI_LOC_POLYMORPH) then
						return true
					end
				end
				ARI_ResetUnitAuraOne("target")
				ClearTarget()
			end
		end
	end

	AssistUnit(leader)

	if not UnitCanAttack("player", "target") then
		ClearTarget()
		return false
	end

	if UnitIsDead("target") then
		ClearTarget()
		return false
	end

	if ARI_UseSpells then
		if ARI_CastAction(ARI_LOC_FROSTBOLT) then
			return true
		end
		if ARI_CastAction(ARI_LOC_FIREBALL) then
			return true
		end
	end

	-- check mana
	local manaPercent = UnitMana("player") / UnitManaMax("player")
	if manaPercent < 0.20 then
		if not ARI_IsActionActive(ARI_LOC_SHOOT) then
			if ARI_CastAction(ARI_LOC_SHOOT) then
				return true
			end
		end
	end
	return false
end

function ARI_HealMage()
	if UnitIsDead("player") then
		return true
	end

	-- force view and refresh spell slots
	SetView(5)
	ARI_CheckActionSlots()
	ARI_ResetUnitAura()

	if ARI_UnitHasDebuff("player", ARI_LOC_MANDOKIR_THREATENING_GAZE) then
		if ARI_IsActionActive(ARI_LOC_SHOOT) then
			if ARI_CastAction(ARI_LOC_SHOOT) then
				return true
			end
		end
		if ARI_PlayerIsCasting() then
			SpellStopCasting()
		end
		return true
	end

	if ARI_PlayerIsCasting() then
		return true
	end

	-- check combat
	local inCombat = UnitAffectingCombat("player")
	if inCombat then
		-- check mana (in combat)
		local manaPercent = UnitMana("player") / UnitManaMax("player")
		if ARI_UsePotions then
			if manaPercent < 0.66 then
				if ARI_UnitCastAction("player", ARI_LOC_MAJOR_MANA_POTION) then
					--return true
				end
			end
		end	
		if manaPercent < 0.20 then
			if ARI_CastAction(ARI_LOC_EVOCATION) then
				return true
			end
		end
	end

	-- check armor
	if ARI_IsActionValid(ARI_LOC_MAGE_ARMOR) then
		if not ARI_UnitHasBuff("player", ARI_LOC_MAGE_ARMOR) then
			if ARI_CastAction(ARI_LOC_MAGE_ARMOR) then
				return true
			end
		end
	elseif ARI_IsActionValid(ARI_LOC_ICE_ARMOR) then
		if not ARI_UnitHasBuff("player", ARI_LOC_ICE_ARMOR) then
			if ARI_CastAction(ARI_LOC_ICE_ARMOR) then
				return true
			end
		end
	elseif ARI_IsActionValid(ARI_LOC_FROST_ARMOR) then
		if not ARI_UnitHasBuff("player", ARI_LOC_FROST_ARMOR) then
			if ARI_CastAction(ARI_LOC_FROST_ARMOR) then
				return true
			end
		end
	end

	-- refresh units in party
	ARI_UpdateUnitNames()

	local focus = ARI_FindFocus()
	local leader = ARI_FindLeader()

	-- check debuffs (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealMage_CheckDebuffsExt(focus) then
			return true
		end
	end
	if ARI_HealMage_CheckDebuffsExt(leader) then
		return true
	end

	-- check debuffs
	for i,unit in pairs(ARI_UnitNamesBuff) do
		if ARI_HealMage_CheckDebuffs(unit) then
			return true
		end
	end

	-- check buffs
	if not UnitAffectingCombat("player") then
		local deadCount = table.getn(ARI_UnitNamesDead)
		if deadCount == 0 then
			for i,unit in pairs(ARI_UnitNamesBuff) do
				if ARI_HealMage_CheckBuffs(unit) then
					return true
				end
			end
		end
	end

	return false
end

function ARI_HealMage_CheckDebuffsExt(unit)
	if not UnitIsDead(unit) then
		-- check follow/buff distance (28m/30m)
		if CheckInteractDistance(unit, 4) then
			ARI_HealMage_CheckDebuffs(unit)
		end
	end
end

function ARI_HealMage_CheckDebuffs(unit)
	if ARI_IsActionValid(ARI_LOC_REMOVE_LESSER_CURSE) then
		if ARI_UnitHasDebuffType(unit, ARI_LOC_CURSE) then
			if ARI_UnitCastAction(unit, ARI_LOC_REMOVE_LESSER_CURSE) then
				return true
			end
		end
	end

	return false
end

function ARI_HealMage_CheckBuffs(unit)
	if not ARI_UnitHasBuff(unit, ARI_LOC_PHASE_SHIFT) then
		if ARI_IsActionValid(ARI_LOC_ARCANE_INTELLECT) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_ARCANE_BRILLIANCE) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_ARCANE_INTELLECT) then
					if ARI_UnitCastAction(unit, ARI_LOC_ARCANE_BRILLIANCE) then
						return true
					end
					if ARI_UnitCastAction(unit, ARI_LOC_ARCANE_INTELLECT) then
						return true
					end
				end
			end
		end
	end

	return false
end