function ARI_PaladinAssist()
	-- this is a placeholder for now
	return ARI_HealPaladin()
end

function ARI_HealPaladin()
	if UnitIsDead("player") then
		return true
	end

	-- force view and refresh spell slots
	SetView(5)
	ARI_CheckActionSlots()
	ARI_ResetUnitAura()
	
	if ARI_UnitHasDebuff("player", ARI_LOC_MANDOKIR_THREATENING_GAZE) then
		if ARI_PlayerIsCasting() then
			SpellStopCasting()
		end
		return true
	end

	if ARI_CheckManaConserve() then
		return true
	end

	-- refresh units in party
	ARI_UpdateUnitNames()

	-- check combat
	local inCombat = UnitAffectingCombat("player")
	if not inCombat then
		-- check for dead (out of combat)
		if ARI_IsActionValid(ARI_LOC_REDEMPTION) then
			for i,unit in pairs(ARI_UnitNamesDead) do
				if UnitIsDead(unit) then
					if ARI_UnitCastAction(unit, ARI_LOC_REDEMPTION) then
						return true
					end
				end
			end
		end
	else
		-- check mana (in combat)
		if ARI_UsePotions then
			local manaPercent = UnitMana("player") / UnitManaMax("player")
			if manaPercent < 0.66 then
				if ARI_UnitCastAction("player", ARI_LOC_MAJOR_MANA_POTION) then
					--return true
				end
			end
		end
	end

	-- check self-buffs
	if not ARI_UnitHasBuff("player", ARI_LOC_RETRIBUTION_AURA) then
		if not ARI_UnitHasBuff("player", ARI_LOC_DEVOTION_AURA) then
			if ARI_CastAction(ARI_LOC_RETRIBUTION_AURA) then
				return true
			end
			if ARI_CastAction(ARI_LOC_DEVOTION_AURA) then
				return true
			end
		end
	end

	local leader = ARI_FindLeader()
	local focus = ARI_FindFocus()

	-- check debuffs (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealPaladin_CheckDebuffsExt(focus) then
			return true
		end
	end
	if ARI_HealPaladin_CheckDebuffsExt(leader) then
		return true
	end

	-- check health (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealPaladin_CheckHealthExt(focus) then
			return true
		end
	end
	if ARI_HealPaladin_CheckHealthExt(leader) then
		return true
	end

	-- check debuffs
	for i,unit in pairs(ARI_UnitNamesBuff) do
		if ARI_HealPaladin_CheckDebuffs(unit) then
			return true
		end
	end

	-- check health (others)
	for i,unit in pairs(ARI_UnitNamesHeal) do
		if ARI_HealPaladin_CheckHealth(unit) then
			return true
		end
	end

	-- check buffs
	if not inCombat then
		local deadCount = table.getn(ARI_UnitNamesDead)
		if deadCount == 0 then
			for i,unit in pairs(ARI_UnitNamesBuff) do
				if ARI_HealPaladin_CheckBuffs(unit) then
					return true
				end
			end
		end
	else
		for i,unit in pairs(ARI_UnitNamesBuff) do
			if ARI_HealPaladin_CheckBuffs(unit) then
				return true
			end
		end
	end

	return false
end

function ARI_HealPaladin_CheckHealthExt(unit)
	if not UnitIsDead(unit) then
		local inRange = true
		if ARI_CheckInteractDistanceExtension then
			-- check heal distance (40m)
			inRange = CheckInteractDistance(unit, 7)
		end
		if inRange then
			ARI_HealPaladin_CheckHealth(unit)
		end
	end
end

function ARI_HealPaladin_CheckHealth(unit)
	local healthPercent = UnitHealth(unit) / UnitHealthMax(unit)
	if healthPercent <= 0.6 then
		if ARI_UnitCastAction(unit, ARI_LOC_HOLY_LIGHT) then
			ARI_SetManaConserve(unit, 0.8)
			return true
		end
	end
	if healthPercent <= 0.75 then
		if ARI_UnitCastAction(unit, ARI_LOC_FLASH_OF_LIGHT) then
			ARI_SetManaConserve(unit, 0.9)
			return true
		end
	end

	return false
end

function ARI_HealPaladin_CheckDebuffsExt(unit)
	if not UnitIsDead(unit) then
		-- check follow/buff distance (28m/30m)
		if CheckInteractDistance(unit, 4) then
			ARI_HealPaladin_CheckDebuffs(unit)
		end
	end
end

function ARI_HealPaladin_CheckDebuffs(unit)
	if ARI_IsActionValid(ARI_LOC_CLEANSE) or ARI_IsActionValid(ARI_LOC_PURIFY) then
		if ARI_UnitHasDebuffType(unit, ARI_LOC_POISON) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_ABOLISH_POISON) then
				if ARI_UnitCastAction(unit, ARI_LOC_CLEANSE) then
					return true
				end
				if ARI_UnitCastAction(unit, ARI_LOC_PURIFY) then
					return true
				end
			end
		end

		if ARI_UnitHasDebuffType(unit, ARI_LOC_DISEASE) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_ABOLISH_DISEASE) then
				if ARI_UnitCastAction(unit, ARI_LOC_CLEANSE) then
					return true
				end
				if ARI_UnitCastAction(unit, ARI_LOC_PURIFY) then
					return true
				end
			end
		end

		if ARI_UnitHasDebuffType(unit, ARI_LOC_MAGIC) then
			if ARI_UnitCastAction(unit, ARI_LOC_CLEANSE) then
				return true
			end
		end
	end

	return false
end

function ARI_HealPaladin_CheckBuffs(unit)
	if not ARI_UnitHasBuff(unit, ARI_LOC_PHASE_SHIFT) then
		if ARI_IsActionValid(ARI_LOC_BLESSING_OF_KINGS) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_KINGS) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_KINGS) then
					if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_KINGS) then
						return true
					end
					if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_KINGS) then
						return true
					end
				end
			end
		else
			if UnitClass(unit) == ARI_LOC_WARRIOR then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_MIGHT) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_MIGHT) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_MIGHT) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_MIGHT) then
							return true
						end
					end
				end
			end

			if UnitClass(unit) == ARI_LOC_HUNTER then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_WISDOM) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_WISDOM) then
							return true
						end
					end
				end
			end

			if UnitClass(unit) == ARI_LOC_ROGUE then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_MIGHT) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_MIGHT) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_MIGHT) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_MIGHT) then
							return true
						end
					end
				end
			end

			if UnitClass(unit) == ARI_LOC_PALADIN then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_WISDOM) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_WISDOM) then
							return true
						end
					end
				end
			end

			if UnitClass(unit) == ARI_LOC_SHAMAN then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_WISDOM) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_WISDOM) then
							return true
						end
					end
				end
			end

			if UnitClass(unit) == ARI_LOC_DRUID then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_WISDOM) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_WISDOM) then
							return true
						end
					end
				end
			end

			if UnitClass(unit) == ARI_LOC_PRIEST then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_WISDOM) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_WISDOM) then
							return true
						end
					end
				end
			end

			if UnitClass(unit) == ARI_LOC_MAGE then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_WISDOM) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_WISDOM) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_WISDOM) then
							return true
						end
					end
				end
			end

			if UnitClass(unit) == ARI_LOC_WARLOCK then
				if not ARI_UnitHasBuff(unit, ARI_LOC_GREATER_BLESSING_OF_SALVATION) then
					if not ARI_UnitHasBuff(unit, ARI_LOC_BLESSING_OF_SALVATION) then
						if ARI_UnitCastAction(unit, ARI_LOC_GREATER_BLESSING_OF_SALVATION) then
							return true
						end
						if ARI_UnitCastAction(unit, ARI_LOC_BLESSING_OF_SALVATION) then
							return true
						end
					end
				end
			end 
		end
	end

	return false
end