function ARI_PriestAssist()
	if ARI_HealPriest() then
		return true
	end

	local leader = ARI_FindLeader()

	AssistUnit(leader)

	if not UnitCanAttack("player", "target") then
		ClearTarget()
		return false
	end

	if UnitIsDead("target") then
		ClearTarget()
		return false
	end

	if ARI_PlayerIsCasting() then
		return true
	end

	if ARI_IsActionValid(ARI_LOC_SHADOWFORM) then
		if not ARI_UnitHasBuff("player", ARI_LOC_SHADOWFORM) then
			if ARI_CastAction(ARI_LOC_SHADOWFORM) then
				return true
			end
		end
	end

	if ARI_IsActionValid(ARI_LOC_SHADOW_WORD_PAIN) then
		if not ARI_IsImmune(ARI_LOC_SHADOW_WORD_PAIN, UnitName("target")) then
			if not ARI_UnitHasDebuff("target", ARI_LOC_SHADOW_WORD_PAIN) then
				if ARI_CastAction(ARI_LOC_SHADOW_WORD_PAIN) then
					return true
				end
			end
		end
	end

	if ARI_IsActionValid(ARI_LOC_VAMPIRIC_EMBRACE) then
		if not ARI_IsImmune(ARI_LOC_VAMPIRIC_EMBRACE, UnitName("target")) then
			if not ARI_UnitHasDebuff("target", ARI_LOC_VAMPIRIC_EMBRACE) then
				if ARI_CastAction(ARI_LOC_VAMPIRIC_EMBRACE) then
					return true
				end
			end
		end
	end

	if ARI_IsActionValid(ARI_LOC_MIND_BLAST) then
		if not ARI_IsImmune(ARI_LOC_MIND_BLAST, UnitName("target")) then
			if not ARI_UnitHasDebuff("target", ARI_LOC_MIND_BLAST) then
				if ARI_CastAction(ARI_LOC_MIND_BLAST) then
					return true
				end
			end
		end
	end

	if ARI_IsActionValid(ARI_LOC_MIND_FLAY) then
		if not ARI_IsImmune(ARI_LOC_MIND_FLAY, UnitName("target")) then
			if not ARI_UnitHasDebuff("target", ARI_LOC_MIND_FLAY) then
				if ARI_CastAction(ARI_LOC_MIND_FLAY) then
					return true
				end
			end
		end
	end

	if ARI_IsActionValid(ARI_LOC_SMITE) then
		if not ARI_IsImmune(ARI_LOC_SMITE, UnitName("target")) then
			if ARI_CastAction(ARI_LOC_SMITE) then
				return true
			end
		end
	end

	-- check mana
	local manaPercent = UnitMana("player") / UnitManaMax("player")
	if manaPercent < 0.20 then
		if not ARI_IsActionActive(ARI_LOC_SHOOT) then
			if ARI_CastAction(ARI_LOC_SHOOT) then
				return true
			end
		end
	end

	return false
end

function ARI_HealPriest()
	if UnitIsDead("player") then
		return true
	end

	-- force view and refresh spell slots
	SetView(5)
	ARI_CheckActionSlots()
	ARI_ResetUnitAura()

	if ARI_UnitHasDebuff("player", ARI_LOC_MANDOKIR_THREATENING_GAZE) then
		if ARI_IsActionActive(ARI_LOC_SHOOT) then
			if ARI_CastAction(ARI_LOC_SHOOT) then
				return true
			end
		end
		if ARI_PlayerIsCasting() then
			SpellStopCasting()
		end
		return true
	end

	if ARI_CheckManaConserve() then
		return true
	end

	-- refresh units in party
	ARI_UpdateUnitNames()

	-- check combat
	local inCombat = UnitAffectingCombat("player")
	if not inCombat then
		-- check for dead (out of combat)
		if not UnitAffectingCombat("player") then
			if ARI_IsActionValid(ARI_LOC_RESURRECTION) then
				for i,unit in pairs(ARI_UnitNamesDead) do
					if UnitIsDead(unit) then
						if ARI_UnitCastAction(unit, ARI_LOC_RESURRECTION) then
							return true
						end
					end
				end
			end
		end
	else
		-- check mana (in combat)
		if ARI_UsePotions then
			local manaPercent = UnitMana("player") / UnitManaMax("player")
			if manaPercent < 0.66 then
				if ARI_UnitCastAction("player", ARI_LOC_MAJOR_MANA_POTION) then
					--return true
				end
			end
		end
	end

	local leader = ARI_FindLeader()
	local focus = ARI_FindFocus()

	-- check debuffs (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealPriest_CheckDebuffsExt(focus) then
			return true
		end
		if ARI_HealPriest_CheckFearWard(focus) then
			return true
		end
	end
	if ARI_HealPriest_CheckDebuffsExt(leader) then
		return true
	end

	-- check health (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealPriest_CheckHealthExt(focus) then
			return true
		end
	end
	if ARI_HealPriest_CheckHealthExt(leader) then
		return true
	end

	-- check debuffs
	for i,unit in pairs(ARI_UnitNamesBuff) do
		if ARI_HealPriest_CheckDebuffs(unit) then
			return true
		end
	end

	-- check health (others)
	for i,unit in pairs(ARI_UnitNamesHeal) do
		if ARI_HealPriest_CheckHealth(unit) then
			return true
		end
	end

	-- check buffs (out of combat)
	if not inCombat then
		local deadCount = table.getn(ARI_UnitNamesDead)
		if deadCount == 0 then
			for i,unit in pairs(ARI_UnitNamesBuff) do
				if ARI_HealPriest_CheckBuffs(unit) then
					return true
				end
			end
		end
	end

	return false
end

function ARI_HealPriest_CheckFearWard(unit)
	if not UnitIsDead(unit) then
		if ARI_IsActionValid(ARI_LOC_FEAR_WARD) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_FEAR_WARD) then
				if ARI_UnitCastAction(unit, ARI_LOC_FEAR_WARD) then
					return true
				end
			end
		end
	end

	return false
end

function ARI_HealPriest_CheckHealthExt(unit)
	if not UnitIsDead(unit) then
		local inRange = true
		if ARI_CheckInteractDistanceExtension then
			-- check heal distance (40m)
			inRange = CheckInteractDistance(unit, 7)
		end
		if inRange then
			ARI_HealPriest_CheckHealth(unit)
		end
	end
end

function ARI_HealPriest_CheckHealth(unit)
	local healthPercent = UnitHealth(unit) / UnitHealthMax(unit)
	if healthPercent <= 0.4 then
		if ARI_IsActionValid(ARI_LOC_FLASH_HEAL) then
			if ARI_UnitCastAction(unit, ARI_LOC_FLASH_HEAL) then
				ARI_SetManaConserve(unit, 0.8)
				return true
			end
		end
	end

	if healthPercent <= 0.7 then
		if ARI_IsActionValid(ARI_LOC_HEAL) or ARI_IsActionValid(ARI_LOC_LESSER_HEAL) then
			if ARI_UnitCastAction(unit, ARI_LOC_HEAL) then
				ARI_SetManaConserve(unit, 0.9)
				return true
			end
			if ARI_UnitCastAction(unit, ARI_LOC_LESSER_HEAL) then
				ARI_SetManaConserve(unit, 0.9)
				return true
			end
		end
	end

	if ARI_UseHots then
		if healthPercent <= 0.8 then
			if ARI_IsActionValid(ARI_LOC_RENEW) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_RENEW) then
					if ARI_UnitCastAction(unit, ARI_LOC_RENEW) then
						return true
					end
				end
			end
		end
	end

	return false
end

function ARI_HealPriest_CheckDebuffsExt(unit)
	if not UnitIsDead(unit) then
		-- check follow/buff distance (28m/30m)
		if CheckInteractDistance(unit, 4) then
			ARI_HealPriest_CheckDebuffs(unit)
		end
	end
end

function ARI_HealPriest_CheckDebuffs(unit)
	if ARI_IsActionValid(ARI_LOC_DISPEL_MAGIC) then
		if ARI_UnitHasDebuffType(unit, ARI_LOC_MAGIC) then
			if ARI_UnitCastAction(unit, ARI_LOC_DISPEL_MAGIC) then
				return true
			end
		end
	end

	if ARI_IsActionValid(ARI_LOC_ABOLISH_DISEASE) or ARI_IsActionValid(ARI_LOC_CURE_DISEASE) then
		if ARI_UnitHasDebuffType(unit, ARI_LOC_DISEASE) then
			if ARI_IsActionValid(ARI_LOC_ABOLISH_DISEASE) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_ABOLISH_DISEASE) then
					if ARI_UnitCastAction(unit, ARI_LOC_ABOLISH_DISEASE) then
						return true
					end
				else
					return false
				end
			end
			if ARI_UnitCastAction(unit, ARI_LOC_CURE_DISEASE) then
				return true
			end
		end
	end

	return false
end

function ARI_HealPriest_CheckBuffs(unit)
	if not ARI_UnitHasBuff(unit, ARI_LOC_PHASE_SHIFT) then
		if ARI_IsActionValid(ARI_LOC_POWER_WORD_FORTIDUDE) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_PRAYER_OF_FORTITUDE) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_POWER_WORD_FORTIDUDE) then
					if ARI_UnitCastAction(unit, ARI_LOC_PRAYER_OF_FORTITUDE) then
						return true
					end
					if ARI_UnitCastAction(unit, ARI_LOC_POWER_WORD_FORTIDUDE) then
						return true
					end
				end
			end
		end

		if ARI_IsActionValid(ARI_LOC_DIVINE_SPIRIT) then
			if not ARI_UnitHasBuff(unit, ARI_LOC_PRAYER_OF_SPIRIT) then
				if not ARI_UnitHasBuff(unit, ARI_LOC_DIVINE_SPIRIT) then
					if ARI_UnitCastAction(unit, ARI_LOC_PRAYER_OF_SPIRIT) then
						return true
					end
					if ARI_UnitCastAction(unit, ARI_LOC_DIVINE_SPIRIT) then
						return true
					end
				end
			end
		end
	end

	return false
end