function ARI_ShamanAssist()
	if ARI_HealShaman() then
		return true
	end

	local leader = ARI_FindLeader()

	AssistUnit(leader)

	if not UnitCanAttack("player", "target") then
		ClearTarget()
		return false
	end

	if UnitIsDead("target") then
		ClearTarget()
		return false
	end

	if CheckInteractDistance("target", 3) then
		-- melee range
		if not ARI_IsActionActive(ARI_LOC_ATTACK) then
			if ARI_CastAction(ARI_LOC_ATTACK) then
				return true
			end
		end
	else
		if ARI_FindActionSlot(ARI_LOC_LIGHTNING_BOLT) ~= nil then
			if not ARI_IsImmune(ARI_LOC_LIGHTNING_BOLT, UnitName("target")) then
				if ARI_CastAction(ARI_LOC_LIGHTNING_BOLT) then
					return true
				end
			end
		end
	end

	return false
end

function ARI_HealShaman()
	if UnitIsDead("player") then
		return true
	end

	-- force view and refresh spell slots
	SetView(5)
	ARI_CheckActionSlots()
	ARI_ResetUnitAura()

	if ARI_UnitHasDebuff("player", ARI_LOC_MANDOKIR_THREATENING_GAZE) then
		if ARI_PlayerIsCasting() then
			SpellStopCasting()
		end
		return true
	end

	if ARI_CheckManaConserve() then
		return true
	end

	-- refresh units in party
	ARI_UpdateUnitNames()

		-- check combat
	local inCombat = UnitAffectingCombat("player")
	if not inCombat then
		-- check for dead (out of combat)
		if not UnitAffectingCombat("player") then
			if ARI_IsActionValid(ARI_LOC_ANCESTRAL_SPIRIT) then
				for i,unit in pairs(ARI_UnitNamesDead) do
					if UnitIsDead(unit) then
						if ARI_UnitCastAction(unit, ARI_LOC_ANCESTRAL_SPIRIT) then
							return true
						end
					end
				end
			end
		end
	else
		-- check mana (in combat)
		if ARI_UsePotions then
			local manaPercent = UnitMana("player") / UnitManaMax("player")
			if manaPercent < 0.66 then
				if ARI_UnitCastAction("player", ARI_LOC_MAJOR_MANA_POTION) then
					--return true
				end
			end
		end
	end

	local leader = ARI_FindLeader()
	local focus = ARI_FindFocus()

	-- check debuffs (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealShaman_CheckDebuffsExt(focus) then
			return true
		end
	end
	if ARI_HealShaman_CheckDebuffsExt(leader) then
		return true
	end

	-- check health (priorize focus & leader)
	if focus ~= nil then
		if ARI_HealShaman_CheckHealthExt(focus) then
			return true
		end
	end
	if ARI_HealShaman_CheckHealthExt(leader) then
		return true
	end

	-- check debuffs (others)
	for _, unit in pairs(ARI_UnitNamesBuff) do
		if ARI_HealShaman_CheckDebuffs(unit) then
			return true
		end
	end
	
	-- check health (others)
	for _, unit in pairs(ARI_UnitNamesHeal) do
		if ARI_HealShaman_CheckHealth(unit) then
			return true
		end
	end

	-- check buffs
	if ARI_HealShaman_CheckBuffs("player") then
		return true
	end
	return false
end

function ARI_HealShaman_CheckHealthExt(unit)
	if not UnitIsDead(unit) then
		local inRange = true
		if ARI_CheckInteractDistanceExtension then
			-- check heal distance (40m)
			inRange = CheckInteractDistance(unit, 7)
		end
		if inRange then
			ARI_HealShaman_CheckHealth(unit)
		end
	end
end

function ARI_HealShaman_CheckHealth(unit)
	local healthPercent = UnitHealth(unit) / UnitHealthMax(unit)
	if healthPercent <= 0.4 then
		if ARI_IsActionValid(ARI_LOC_LESSER_HEALING_WAVE) then
			if ARI_UnitCastAction(unit, ARI_LOC_LESSER_HEALING_WAVE) then
				ARI_SetManaConserve(unit, 0.7)
				return true
			end
		end
	end

	if healthPercent <= 0.75 then
		if ARI_IsActionValid(ARI_LOC_HEALING_WAVE) then
			if ARI_UnitCastAction(unit, ARI_LOC_HEALING_WAVE) then
				ARI_SetManaConserve(unit, 0.9)
				return true
			end
		end
	end

	return false
end

function ARI_HealShaman_CheckDebuffsExt(unit)
	if not UnitIsDead(unit) then
		-- check follow/buff distance (28m/30m)
		if CheckInteractDistance(unit, 4) then
			ARI_HealShaman_CheckDebuffs(unit)
		end
	end
end

function ARI_HealShaman_CheckDebuffs(unit)
	if ARI_IsActionValid(ARI_LOC_CURE_POISON) then
		if ARI_UnitHasDebuffType(unit, ARI_LOC_POISON) then
			if ARI_UnitCastAction(unit, ARI_LOC_CURE_POISON) then
				return true
			end
		end
	end

	if ARI_IsActionValid(ARI_LOC_CURE_DISEASE) then
		if ARI_UnitHasDebuffType(unit, ARI_LOC_DISEASE) then
			if ARI_UnitCastAction(unit, ARI_LOC_CURE_DISEASE) then
				return true
			end
		end
	end

	return false
end

function ARI_HealShaman_CheckBuffs(unit)
	-- earth totems
	if ARI_IsActionValid(ARI_LOC_STRENGTH_OF_EARTH_TOTEM) then
		if not ARI_UnitHasBuff(unit, ARI_LOC_STRENGTH_OF_EARTH) then
			if ARI_UnitCastAction(unit, ARI_LOC_STRENGTH_OF_EARTH_TOTEM) then
				return true
			end
		end
	elseif ARI_IsActionValid(ARI_LOC_STONESKIN_TOTEM) then
		if not ARI_UnitHasBuff(unit, ARI_LOC_STONESKIN) then
			if ARI_UnitCastAction(unit, ARI_LOC_STONESKIN_TOTEM) then
				return true
			end
		end
	end

	-- air totems
	if ARI_IsActionValid(ARI_LOC_GRACE_OF_AIR_TOTEM) then
		if not ARI_UnitHasBuff(unit, ARI_LOC_GRACE_OF_AIR) then
			if ARI_UnitCastAction(unit, ARI_LOC_GRACE_OF_AIR_TOTEM) then
				return true
			end
		end
	elseif ARI_IsActionValid(ARI_LOC_WINDFURY_TOTEM) then
		hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
		if hasMainHandEnchant ~= 1 then
			if ARI_UnitCastAction(unit, ARI_LOC_WINDFURY_TOTEM) then
				return true
			end
		end
	end

	-- water totems
	if ARI_IsActionValid(ARI_LOC_MANA_SPRING_TOTEM) then
		if not ARI_UnitHasBuff(unit, ARI_LOC_MANA_SPRING) then
			if ARI_UnitCastAction(unit, ARI_LOC_MANA_SPRING_TOTEM) then
				return true
			end
		end
	elseif ARI_IsActionValid(ARI_LOC_HEALING_STREAM_TOTEM) then
		if not ARI_UnitHasBuff(unit, ARI_LOC_HEALING_STREAM) then
			if ARI_UnitCastAction(unit, ARI_LOC_HEALING_STREAM_TOTEM) then
				return true
			end
		end
	end

	-- fire totems
	if ARI_IsActionValid(ARI_LOC_FLAMETONGUE_TOTEM)  then
		hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
		if hasMainHandEnchant ~= 1 then
			if ARI_UnitCastAction(unit, ARI_LOC_FLAMETONGUE_TOTEM) then
				return true
			end
		end
	end

	return false
end
